#ifndef FILELOADER_H
#define FILELOADER_H

#define FILE_HEADER "#STACKED"

#include <QObject>
#include <QTextStream>
#include <QFile>
#include <dataset.h>

/*!
Responsible for loading data sets from file or generating random data to use.
*/
class DataLoader : public QObject
{
	Q_OBJECT

public:
	/*!
	Default Qt-style constructor.
	Initializes the random seed for \ref RandomData
	*/
	explicit DataLoader(QObject *parent = 0);
	~DataLoader();

	/*!
	Loads a data set from the specified file path. 
	The file format is basically a CSV, with tuples of (label,data) entries.
	Each row is an element of the time series (usually month, etc).
	The first line of the file has to be the string \c #STACKED
	\param path The path to the data file
	\param dataset the \ref DataSet to load into
	\return true if loading succeeded, false on error
	*/
	bool LoadData(const QString& path, DataSet& dataset) const;
	/*!
	Generates a random dataset of \c numItems layers. It will have a time series of length 12.
	The dataset is created by randomly placing some gaussian curves on each layer.
	\param numItems the number of layers to create
	\param dataset the \ref DataSet to write into
	*/
	void RandomData(int numItems, DataSet& dataset) const;

private:
	void createRandomItemData(DataItem& item) const;
	void setMinMaxValues(DataSet& dataset) const;
};

#endif // FILELOADER_H
