#include "stdafx.h"
#include "graphcolorizer.h"

const char* const GraphColorizer::coloringAttribute_str[ColoringAttribute::SIZE_OF_ENUM] ={"OnsetTime", "Volatility"};

GraphColorizer::GraphColorizer(QObject *parent)
	: QObject(parent)
{
	this->m2DColoring = false;
}

GraphColorizer::~GraphColorizer()
{
}

void GraphColorizer::SetColors(QColor const beginColor, QColor const endColor)
{
	if(beginColor != nullptr)
	{
		this->mBeginColor = beginColor;
	}

	if(endColor != nullptr)
	{
		this->mEndColor = endColor;
	}
}

void GraphColorizer::Set2DColoring(bool enable)
{
	this->m2DColoring = enable;
}

void GraphColorizer::Colorize(DataSet& const dataset, ColoringAttribute const attribute)
{
	float ratio;

	const DataPtrListRef dat = dataset.getSortedData();
	for (auto it = dat.begin(); it != dat.end(); it++)
	{
		DataItem& item = **it;

		ratio = calculateRatio(dataset, item, attribute);
		int alpha = 255;
		
		if(this->m2DColoring)
		{
			float alpharatio = (float)(item.sum - dataset.getMinSum()) / (float)(dataset.getMaxSum()-dataset.getMinSum());
			alpha = alpharatio * 255;

			//convert from 0-255 to 128-255
			alpha = (int)(127.0f + (float(alpha) / float(255) * 128.0f));
		}

		QColor color;
		int red = (1-ratio) * this->mBeginColor.red() + ratio * this->mEndColor.red();
		int green = (1-ratio) * this->mBeginColor.green() + ratio * this->mEndColor.green();
		int blue = (1-ratio) * this->mBeginColor.blue() + ratio * this->mEndColor.blue();
		color.setRgb(red,green,blue,alpha);

		item.color = color;
	}
}

float GraphColorizer::calculateRatio(DataSet& const dataset, DataItem& const dataitem, ColoringAttribute const attribute)
{
	if(attribute == ColoringAttribute::OnsetTime)
	{
		return (float)(dataitem.onsetTime - dataset.getMinOnsetTime()) / (float)(dataset.getMaxOnsetTime()-dataset.getMinOnsetTime());
	}
	else if(attribute == ColoringAttribute::Volatility)
	{
		return (dataitem.volatility - dataset.getMinVolatility()) / (dataset.getMaxVolatility()-dataset.getMinVolatility());
	}

	return 0.0f;
}


