#ifndef GRAPHCOLORIZER_H
#define GRAPHCOLORIZER_H

#include <QObject>
#include "dataset.h"

/*!
	Class provides coloring for the graph layers by using linear interpolation
*/
class GraphColorizer : public QObject
{
	Q_OBJECT

public:
	/*!
	List of attributes which can be used to calculate an appropriate color value
	*/
	enum ColoringAttribute { OnsetTime = 0, Volatility, SIZE_OF_ENUM };

	/*!
	String value of the attributes used to display in the GUI
	*/
	static const char* const coloringAttribute_str[ColoringAttribute::SIZE_OF_ENUM];

	GraphColorizer(QObject *parent);
	~GraphColorizer();

	/*!
	Sets the colors which define the start and end color of the linearly interpolated color gradient used to define the stacked graphs layer colors	
	\param beginColor the start color of the linear gradient
	\param endColor the end color of the linear gradient
	*/
	void SetColors(QColor const beginColor, QColor const endColor);

	/*!
	Sets a flag which enables 2D coloring: alpha value of color changes according to the layours sum value
	\param enable the set flag: true...enable 2D coloring, false...disable 2D coloring
	*/
	void Set2DColoring(bool enable);

	/*!
	Assigns each dataitem a color value according to the set begin/end colors and coloring attribute
	\param dataset the data which should be colorized
	\param attr the attribute which is used to determine the color
	*/
	void Colorize(DataSet& const dataset, ColoringAttribute attr);
private:
	float calculateRatio(DataSet& const dataset, DataItem& const dataitem, ColoringAttribute const attribute);

	QColor mBeginColor;
	QColor mEndColor;
	bool m2DColoring;
};

#endif // GRAPHCOLORIZER_H
