#include "stdafx.h"
#include "graphlayouter.h"

const char* const GraphLayouter::interpolationMode_str[InterpolationMode::SIZE_OF_ENUM] = {"Cubic", "Linear"};

GraphLayouter::GraphLayouter(QObject *parent)
	: QObject(parent)
{

}

GraphLayouter::~GraphLayouter()
{
}

void GraphLayouter::Layout(DataSet& dataset, const InterpolationMode interpMode, const double scaleX, const double scaleY)
{
	QVector<double> cumHeight;

	this->calculateBaseLine(dataset, cumHeight);

	double originX = 0.0;
	double originY = 0.0;

	const DataPtrListRef dat = dataset.getSortedData();
	for (auto it = dat.begin(); it != dat.end(); it++)
	{
		DataItem& item = **it;
		EntryVector& timeSeries = item.data;


		int first = item.onsetTime > 0 ? item.onsetTime - 1 : 0;
		int last = item.lastTime < (timeSeries.size()-1) ? item.lastTime +1 : (timeSeries.size()-1);
		QPainterPath cumPath;
		moveTo(cumPath, originX, originY, first, cumHeight[first],scaleX, scaleY);
		for (int i = item.onsetTime; i <= last; i++) 
		{
			double pX = (double)i;
			double pY = cumHeight[i];

			pathTo(cumPath, interpMode, originX, originY, pX, pY,scaleX, scaleY);
		}

		QPainterPath path;
		moveTo(path, originX, originY, first, cumHeight[first],scaleX, scaleY);
		for (int i = first; i <= last; i++) 
		{
			double pX = (double)i;
			double pY = cumHeight[i]+timeSeries.at(i);

			pathTo(path, interpMode, originX, originY, pX, pY,scaleX, scaleY);

			if(i == item.maxIdx)
			{
				QPointF offset(0,0);
				if(i == 0)
				{
					//maximum at 0, move a bit right
					offset.setX(scaleX/2);
				}
				else if (i==timeSeries.size()-1)
				{
					//maximum at last pt, move a bit left
					offset.setX(-scaleX/2);
				}
				item.labelPos = QPointF(pX * scaleX, -(cumHeight[i] + timeSeries.at(i) / 2.0f) * scaleY) + offset;
			}

			cumHeight[i] += timeSeries.at(i);			
		}
		pathTo(path, interpMode, originX, originY, last, cumHeight[last]-timeSeries.at(last),scaleX, scaleY);
		path.addPath(cumPath.toReversed()); //add bottom line
		//path.closeSubpath();	
		//REMOVED BECAUSE IT SOMETIMES DESTROYS LINEAR INTERPOLATED PATHS....
		//path = path.simplified(); //remove duplicate segments

		item.layout.path = path;
	}
}

void GraphLayouter::pathTo(QPainterPath& path, InterpolationMode interpMode, double& originX, double& originY, double pX, double pY, double xScale, double yScale)
{
	double cX1, cY1, cX2, cY2;

	switch(interpMode)
	{
	case InterpolationMode::Cubic:
		cX1 = originX + ((pX - originX) / 2.0);
		cY1 = originY;
		cX2 = cX1;
		cY2 = pY;

		path.cubicTo(cX1 * xScale, -cY1 * yScale, cX2 * xScale, -cY2 * yScale, pX * xScale, -pY * yScale);
		break;
	case InterpolationMode::Linear:
		path.lineTo(pX*xScale,-pY*yScale);
		break;
	}

	originX = pX;
	originY = pY;
}

void GraphLayouter::moveTo(QPainterPath& path, double& originX, double& originY, double pX, double pY, double xScale, double yScale)
{
	path.moveTo(pX * xScale, -pY * yScale);
	originX = pX;
	originY = pY;
}
