#ifndef GRAPHLAYOUTER_H
#define GRAPHLAYOUTER_H

#include <QObject>
#include "dataset.h"
#include "namedobject.h"

/*!
	Derived classes provide layouting functionality. They stack the different graph layers on each other using different algorithms
*/
class GraphLayouter : public QObject, public INamedObject
{
	Q_OBJECT
	Q_INTERFACES(INamedObject)
public:
	/*!
	List of interpolation methods
	*/
	enum InterpolationMode { Cubic = 0, Linear, SIZE_OF_ENUM };

	/*!
	String value of the interpolation methods used to display in the GUI
	*/
	static const char* const interpolationMode_str[InterpolationMode::SIZE_OF_ENUM];

	GraphLayouter(QObject *parent);
	~GraphLayouter();

	/*!
	Calls \ref calculateBaseLine to get a baseline according to the selected layouting algorithm. For each \ref DataItem inside the sorted \ref DataSet a QPainterPath is created
	which is stacked onto this baseline
	\param dataset the data which should be layouted
	\param interpMode the interpolation method used to interpolate between the given data points
	\param scaleX the scale of the graph in x-direction
	\param scaleY the scale of the graph in y-direction
	*/
	void Layout(DataSet& dataset, const InterpolationMode interpMode, const double scaleX, const double scaleY);
private:
	void pathTo(QPainterPath& path, InterpolationMode interpMode, double& originX, double& originY, double pX, double pY, double xScale, double yScale);
	void moveTo(QPainterPath& path, double& originX, double& originY, double pX, double pY, double xScale, double yScale);

	virtual void calculateBaseLine(DataSet& dataset, QVector<double>& baseline) = 0;
};

#endif // GRAPHLAYOUTER_H
