#ifndef GRAPHSORTER_H
#define GRAPHSORTER_H

#include "namedobject.h"
#include "dataset.h"

/*!
	Derived classes provide sorting functionality for the graph layers, usign different algorithms.
*/
class GraphSorter : public QObject, public INamedObject
{
	Q_OBJECT
	Q_INTERFACES(INamedObject)
public:
	/*!
	List of attributes used to sort the dataset
	*/
	enum SortingAttribute { OnsetTimeASC = 0, OnsetTimeDESC, VolatilityASC, VolatilityDESC, SIZE_OF_ENUM };

	/*!
	String value of the sorting attributes used to display in the GUI
	*/
	static const char* const sortingAttribute_str[SortingAttribute::SIZE_OF_ENUM];

	GraphSorter(QObject *parent);
	~GraphSorter();

	/*!
	Sorts the dataset in a preprocessing step according to its dataitems sorting attribute value
	\param data the data which should be sorted
	\param attribute the attribute which is used to sort the data
	*/
	void Sort(DataSet& data, const SortingAttribute attribute);
private:
	virtual void orderLayers(DataSet& data) = 0;

	bool comparator (DataItem a, DataItem b, const SortingAttribute sortAttr);
};

#endif // GRAPHSORTER_H
