#include "stdafx.h"
#include "graphview.h"

GraphView::GraphView(GraphScene* scene, QWidget *parent)
	: QGraphicsView(scene,parent), mUpdateRect(true)
{
	//set some defaults
	this->setBackgroundBrush(Qt::white);

	this->setDragMode(QGraphicsView::ScrollHandDrag);
	this->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
	this->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
	this->setTransformationAnchor(QGraphicsView::ViewportAnchor::AnchorUnderMouse);

	this->setRenderHint(QPainter::Antialiasing);

	connect(this->horizontalScrollBar(),&QScrollBar::valueChanged,this,&GraphView::scrolled);
	connect(this->verticalScrollBar(),&QScrollBar::valueChanged,this,&GraphView::scrolled);
}

GraphView::~GraphView()
{

}

void GraphView::scrolled(int val)
{
	Q_UNUSED(val)
	if(mUpdateRect)
		updateSceneRect();
}

void GraphView::wheelEvent(QWheelEvent* ev)
{
	int delta = ev->angleDelta().y() / 120.0f;

	QTransform trans = this->transform();

	float scale = 0.83333333f;
	if(delta > 0)
		scale = 1.2f;

	for(int i = 0;i<qAbs(delta);++i)
	{
		this->scale(scale,scale);
	}

	updateSceneRect();
	ev->accept();
}

void GraphView::mousePressEvent(QMouseEvent* ev)
{
	QPointF p = this->mapToScene(ev->pos());
	qDebug()<<"Scene pos: "<<p;
	//call base
	QGraphicsView::mousePressEvent(ev);
}

void GraphView::updateSceneRect()
{
	//not 100% accurate, but ok if not updated during resize
    QPointF A = this->mapToScene( QPoint(0,0) );
    QPointF B = this->mapToScene( QPoint( 
        this->viewport()->width(), 
        this->viewport()->height() ));
	mCurSceneRect = QRectF(A,B);
}

void GraphView::resizeEvent(QResizeEvent* ev)
{
	mUpdateRect = false;
	this->fitInView(mCurSceneRect,Qt::AspectRatioMode::KeepAspectRatio);
	mUpdateRect = true;
	//this would notably accumulate the errors
	//updateSceneRect();
}

void GraphView::ResetView()
{
	mUpdateRect = false;
	this->fitInView(this->scene()->sceneRect(),Qt::AspectRatioMode::KeepAspectRatio);
	mUpdateRect = true;
	updateSceneRect();
}